/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME
     Random

   PURPOSE
     interface to random number generator

   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY
     created July, 1996

***************************************************************************/


#ifndef _RANDOM_H
#define _RANDOM_H

#include <iostream.h>
#include <stdlib.h>

// forward decl's
void ran_start(long seed);
void ran_array(long* aa, int n);
void ran_double_start(long seed);
void ran_double_array(double* aa,int n);


/* RandomDouble
 *    interface to random double generator
 */

class RandomDouble {
public:
    RandomDouble(int size) { 
	if (size <= 100) {
	    cerr << "RandomDouble: illegal size. size must >= 100\n";
	    abort();
	}
	cache = new double[cacheSize=size];
	current = end = cache + cacheSize; }
    ~RandomDouble() { delete [] cache; }

    inline const double* get(int num);
    void seed(long s) { ran_double_start(s); }
       
private:
    int      cacheSize;
    double*  cache;
    double*  end;
    double*  current;
};


/* get
 *    return a pointer to num random double
 *
 *   num: number of random doubles requested
 */

inline const double* RandomDouble::get(int num)
{
    double* rd = current;
    if ( current+num >= end ) {
	ran_double_array(cache, cacheSize);
	rd = current = cache;
    }
    current += num;
    return rd;
}




/* RandomInt
 *    interface to random int generator
 */

class RandomInt {
public:
    RandomInt(int size) { 
	if (size <= 100) {
	    cerr << "RandomInt: illegal size. size must >= 100\n";
	    abort();
	}
	cache = new long[cacheSize=size];
	current = end = cache + size ; }
    ~RandomInt() { delete [] cache; }

    inline const long* get(int num);
    void seed(long s) { ran_start(s); }
       
private:
    int    cacheSize;
    long*  cache;
    long*  end;
    long*  current;
};

/* get
 *    return a pointer to num random double
 *
 *   num: number of random doubles requested
 */

inline const long* RandomInt::get(int num)
{
    long* rd = current;
    if ( current+num >= end ) {
	ran_array(cache, cacheSize);
	rd = current = cache;
    }
    current += num;
    return rd;
}

#endif /* _RANDOM_H */
