/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME

   PURPOSE
     
   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY

***************************************************************************/


#include "math++.h"
#include "global.H"
#include "dist.h"
#include "robot.H"
#include "config.H"

/* clearance
 *   compute the dist between the robot and the obstacle
 */

Coord Robot::clearance(Config& q )
{
  Sdist3 d;
  Frame3 currFrame;

  Stats.numColCheck++;

  // convert the C-space coord's to R3 x SO(3)
  q.frame( currFrame );

  // compute distance with obstacles
  Sobj3* obsModel = G.obs.model();
  d = sdist3( handle, &currFrame, obsModel, IFRAME, Ct.distErr);
  return ( q.dist = d.d );
}

    
/* connectPair
 *    test whether the straight line segment between q0 and q1 lies in C-free

 */

Bool Robot::connectPair(Config& q0, Config& q1)
{
  // bring the two quaternions on the same hemisphere
  q0.antipode( q1 );
  Stats.numConnectPair++;
  return connectHelper(q0, q1);
}


/* connectHelper
 *    test whether q0 and q1 are connected by a line segment in C-free by
 *    bi-section until a certain resolution is reached
 */
        
Bool Robot::connectHelper(Config& q0, Config& q1)
{
  Config qm;

  //  test d(q0, q1) < max{ clearance(q0) , clearance(q1)}
  //  if this is true, then q0 and q1 can be connected by a straight line
  //  segment lying in C-free

  Coord dist = wdist(q0, q1, radius());   /* dist of q0,q1 in workspace */
  if ( dist <= q0.dist || dist <= q1.dist )
    return TRUE;

  // check whether the resolution of workspace is reached
  else if ( wdist( q0, q1, maxRadius) <= Ct.WRK_SPC_RES )
    return TRUE;
  else {
    // make recursive call
    qm.midpoint(q0, q1);
    clearance(qm);
    return ( ( qm.dist > Ct.tol )	/* qm is in c-free */
	     &&  connectHelper(q0, qm)  
	     &&  connectHelper(qm, q1) );
  }
}
